/*
 * Copyright (c) 2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/types.h>
#include <stdint.h>
#include <stdbool.h>

#include "crc16.h"

#define CRC_ADJUSTMENT 0xa001
#define CRC_TABLE_SIZE 256


static uint16_t crc_table[CRC_TABLE_SIZE];

/*
 * Initialize the CRC table
 */

void
crc_initialize(void)
{
	int i;
	uint16_t j, k;

	for (i = 0; i < CRC_TABLE_SIZE; i++) {
		k = i;
		crc_table[i] = 0;
		for (j = 0; j < 8; j++) {
			if (((crc_table[i] ^ k) & 1) == 1)
				crc_table[i] = (crc_table[i] >> 1)
				    ^ CRC_ADJUSTMENT;
			else
				crc_table[i] >>= 1;
			k >>= 1;
		}
	}
}

/*
 * Update a CRC16, taking a new byte into account
 */
static uint16_t
crc_calculate_crc16(uint8_t *buffer, size_t len)
{
	int i;
	uint16_t temp, crc = 0;

	for (i = 0; i < len; i++) {
		temp = crc ^ buffer[i];
		crc = (crc >> 8) ^ crc_table[temp & 0x00ff];
	}
	return crc;
}

/*
 * Compute a CRC. Store the result in the last 2 bytes of the buffer
 */
void
crc_compute(uint8_t *buffer, size_t len)
{
	int crc_index = len - 2;
	uint16_t crc = 0;

	crc = crc_calculate_crc16(buffer, crc_index);
	/* save the CRC in the last word */
	buffer[crc_index] = (uint8_t)((crc & 0xff00) >> 8);
	buffer[crc_index+1] = (uint8_t)(crc & 0x00ff);
}

/*
 * Check the CRC stored in the last 2 bytes of the buffer
 */
bool
crc_is_valid(uint8_t *buffer, size_t len)
{
	int crc_index = len - 2;
	uint16_t new_crc = 0, received_crc = 0;

	new_crc = crc_calculate_crc16(buffer, crc_index);
	received_crc = (buffer[crc_index] << 8) & 0xff00;
	received_crc |= buffer[++crc_index] & 0x00ff;

	return (received_crc == new_crc);
}
