/*
 * Copyright (c) 2013 CNRS/LAAS
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>

#include <rmp440/rmp440.h>

#include "io.h"
#include "crc16.h"

struct rmp440_io *
rmp440Init(enum RMP440_INTERFACE_TYPE it, const char *name)
{
	struct rmp440_io *rmp;

	switch (it) {
	case RMP440_INTERFACE_USB:
		rmp = rmpUsbInit(name);
		break;
	case RMP440_INTERFACE_ETHERNET:
		rmp = rmpInetInit(name);
		break;
	case RMP440_INTERFACE_CAN:
		rmp = NULL;
		break;
	case RMP440_INTERFACE_FAKE:
		rmp = rmpFakeInit(name);
		break;
	default:
		fprintf(stderr, "rmp440Init: unknown interface type %d\n", it);
		return NULL;
	}
	/* Initialize CRC16 tables */
	crc_initialize();

	return rmp;
}

ssize_t
rmp440Send(const struct rmp440_io *rmp, const uint8_t *buffer)
{
	return rmp->send(rmp, buffer);
}

ssize_t
rmp440Receive(const struct rmp440_io *rmp, uint8_t *buffer, size_t len)
{
	return rmp->receive(rmp, buffer, len);
}


void
rmp440End(struct rmp440_io *rmp)
{
	rmp->close(rmp);
	free(rmp);
}

int
rmp440SendMsgInt(const struct rmp440_io *rmp, uint16_t cmd, uint32_t v1, uint32_t v2)
{
	uint8_t buf[RMP_NUM_USB_ETH_BYTES];

	buf[0] = (cmd & 0xff00) >> 8;
	buf[1] = (cmd & 0x00ff);

	buf[2] = (v1 & 0xff000000) >> 24;
	buf[3] = (v1 & 0x00ff0000) >> 16;
	buf[4] = (v1 & 0x0000ff00) >> 8;
	buf[5] = (v1 & 0x000000ff);

	buf[6] = (v2 & 0xff000000) >> 24;
	buf[7] = (v2 & 0x00ff0000) >> 16;
	buf[8] = (v2 & 0x0000ff00) >> 8;
	buf[9] = (v2 & 0x000000ff);

	crc_compute(buf, RMP_NUM_USB_ETH_BYTES);

	return rmp440Send(rmp, buf) == RMP_NUM_USB_ETH_BYTES ? 0 : -1;
}

int
rmp440SendMsgIntFloat(const struct rmp440_io *rmp, uint16_t cmd,
    uint32_t v1, double v2)
{
	union {
		uint32_t i;
		float f;
	} u2;

	u2.f = (float)v2;
	return rmp440SendMsgInt(rmp, cmd, v1, u2.i);
}

int
rmp440SendMsgFloat(const struct rmp440_io *rmp, uint16_t cmd,
    double v1, double v2)
{
	union {
		uint32_t i;
		float f;
	} u1, u2;

	u1.f = (float)v1;
	u2.f = (float)v2;
	return rmp440SendMsgInt(rmp, cmd, u1.i, u2.i);
}
